/* EDGE standard header file for graphics programs 
 * Stripped down version 3/11/94 
 */

#ifndef EBERT_H
#define EBERT_H

#include <vector>

using namespace std;

#define RAND_GRID 16  

//#define       ABS(a)          ((a) > 0 ? (a) : -(a))
//double fabs(), sqrt(), sin(), cos(), tan();

class a {
};

struct xyz {
    xyz(){}
    xyz (const float x, const float y, const float z) : x (x), y (y), z (z) {}
    xyz (const xyz &a) : x (a.x), y (a.y), z (a.z) {}

    xyz operator+ (const xyz &a) const {
        return xyz (x + a.x, y + a.y, z + a.z);
    }

    xyz operator- (const xyz &a) const {
        return xyz (x - a.x, y - a.y, z - a.z);
    }

    xyz operator* (const float &a) const {
        return xyz (x * a, y * a, z * a);
    }

    xyz operator/ (const float &a) const {
        return xyz (x / a, y / a, z / a);
    }

    xyz& operator= (const xyz &a) {
        x = a.x; y = a.y; z = a.z;
        return *this;
    }

    xyz& operator+= (const xyz &a) {
        x += a.x; y += a.y; z += a.z;
        return *this;
    }

    xyz& operator-= (const xyz &a) {
        x -= a.x; y -= a.y; z -= a.z;
        return *this;
    }

    xyz& operator*= (const float &a) {
        x *= a; y *= a; z *= a;
        return *this;
    }

    xyz& operator/= (const float &a) {
        x /= a; y /= a; z /= a;
        return *this;
    }

    inline float length() const {
        return sqrtf (x*x + y*y + z*z);
    }

    inline float length2() const {
        return x*x + y*y + z*z;
    }

    inline float normalize() {
        float invLen = 1.f / length();
        x *= invLen;
        y *= invLen;
        z *= invLen;
    }

    inline float dot (const xyz &a) const {
        return x * a.x + y * a.y + z * a.z;
    }

    inline void set (const float _x, const float _y, const float _z) {
        x = _x; y = _y; z = _z;
    }

    inline void set (const float all) {
        x = y = z = all;
    }

    float x, y, z;
};

class MetaPrim {
public:
    MetaPrim (const xyz &_pos, const float _weight) : pos(_pos), weight(abs(_weight)) {}

    virtual float density (const xyz &pt) const = 0;
    virtual void bounds (xyz *minPt, xyz *maxPt, const float &stretch) const = 0;


    xyz pos;
    float weight;
};

class Sphere : public MetaPrim {
public:
    Sphere (const xyz &_pos, const float _r, const float _weight = 1.0)
        : MetaPrim (_pos, _weight),
        r (abs(_r)), rInv (1.f / r),
        r2 (pow (r, 2)), r2Inv (1.f / r2),
        r4 (pow (r, 4)), r4Inv (1.f / r4),
        r6 (pow (r, 6)), r6Inv (1.f / r6)
    {}

    float density (const xyz &pt) const;
    void bounds (xyz *minPt, xyz *maxPt, const float &stretch) const;

    float r; // radius
    float r2; // radius^2
    float r4; // radius^4
    float r6; // radius^6
    float rInv; // 1 / radius
    float r2Inv; // 1 / radius^2
    float r4Inv; // 1 / radius^4
    float r6Inv; // 1 / radius^6
};


class Ellipse : public MetaPrim {
public:
    Ellipse (const xyz &_pos, const float _rx, const float _ry, const float _rz, const float _weight = 1.0)
        : MetaPrim (_pos, _weight),
        r (abs(_rx), abs(_ry), abs(_rz)),
        rmax (max (r.x, max  (r.y, r.z))),
        rmax2 (rmax * rmax),
        rmax2Inv (1 / rmax2),
        rmax4Inv (rmax2Inv * rmax2Inv),
        rmax6Inv (rmax2Inv * rmax4Inv),
        spherize (rmax / r.x, rmax / r.y, rmax / r.z)
    {}

    float density (const xyz &pt) const;
    void bounds (xyz *minPt, xyz *maxPt, const float &stretch) const;

    xyz r;
    float rmax, rmax2, rmax2Inv, rmax4Inv, rmax6Inv;
    xyz spherize; 
};


typedef vector<MetaPrim*> MetaPrimVec;

class MetaClump {
public:
    MetaClump (const float _maxDensity) : maxDensity (_maxDensity)  {}

    ~MetaClump() {
        for (MetaPrimVec::iterator primIt = prims.begin(); primIt != prims.end(); ++primIt)
            delete *primIt;
    }

    void addPrim (MetaPrim *prim) {
        prims.push_back (prim);
    }

    float evaluate (const xyz &pt) const {
        float val = 0;
        for (MetaPrimVec::const_iterator primIt = prims.begin(); primIt != prims.end(); ++primIt)
            val += (*primIt)->density (pt);
        return min (val, maxDensity);
    }

    void bounds (xyz *minPt, xyz *maxPt, const float &stretch) const;

private:

    float maxDensity;
    MetaPrimVec prims;
};
/*
typedef struct xyz_td {
    float x, y, z;
} xyz_td;



typedef struct rgb_td {
    float r, g, b;
} rgb_td;


typedef float tran_mat_td[4][4];

// Insert your geometry definitions for implicits here
typedef union {                       // Primitive Description               
// MetaSphere   *msphere;             // meta-sphere                        
// MetaCylinder *mcylinder;           // meta-cylinder                      
// MetaEllipse  *mellipse;            // meta-ellipse                       
 float   *msphere;                    // place holder 
 float   *mcylinder;
} PrimDesc;


typedef struct meta_ball_td {        // Meta-Ball structure                
  short    type;                     // metaball type (SPHERE, CYLINDER,...
  PrimDesc primdesc;                 // primitive description (geometry)   
  double   weight;                   // weighting factor                   
  double   power;                    // blending factor                    
} MetaBall;

*/
/*
 *****************************************************************************
 *                  FUNCTION FIELD DEFINITIONS
 *****************************************************************************
 */
/*
typedef unsigned char flow_func_type;

typedef struct flow_func_td       
    {
      short         func_type;
      xyz_td        center, axis;
      float         distance;
      float         falloff_start;
      short         falloff_type;
      float         parms[20];
    } flow_func_td;

typedef struct vol_shape_td
    {
      xyz_td         center;
      xyz_td         inv_rad_sq;
      xyz_td         inv_rad;
    } vol_shape_td;

typedef struct vol_td
    {
      short          obj_type;
      float          y_obj_min,    // min & max y per volume             
	             y_obj_max;
      float          x_obj_min,    // min & max x per volume             
	             x_obj_max;
      int            shape_type;   // 0=sphere, 1=box;                   
      int            self_shadow;  // 0=no, 1=yes                        
      xyz_td         scale_obj;   //how to scale obj to get into -1:1 space
      xyz_td         trans_obj;   //how to trans obj to get into -1:1 space
      vol_shape_td   shape;
      int            funct_name;
      rgb_td         color;
      float          amb_coef;
      float          diff_coef;
      float          spec_coef;
      float          spec_power;
      int            illum_type;  // illumination - phong, blinn, c-t, gas 
      int            color_type;  // constant, solid
      float          indx_refrac;
      MetaBall       metaball;
    } vol_td;
*/
void generate_noise();
float cumulus_density (xyz pnt,      // location of point in cloud space 
                       xyz pnt_w,    // location of point in world space 
                       const MetaClump &vol,// volume
                       float param1,  // density scaling 
                       float param2,  // exponent in pow() function
                       float param3,  // blending function value between meta-ball density and product of turbulence density. larger means less turbulent
                       float param4); // turbulence in displacing primitive before evaluation of implicit primitives


#endif
