#include <fstream>
#include <iomanip>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <math.h>
#include <vector>
#include <sstream>
#include "ebert.h"

using namespace std;
/*
void cumulus (const xyz pnt, // 
              float *density, // 
              xyz pnt_w, // 
              // vol? , // 
              float parms1, // density scaling
              float parms2, // exponent in pow() function
              float parms3, // blending function value between meta-ball density and product of turbulence density. larger means less turbulent
              float parms4) // turbulence in displacing primitive before evaluation of implicit primitives
{
}
*/
bool processFiles (ifstream &geomFile, ofstream &denseFile,
                   const float samplesPerUnitLength,
                   const float densityScale,
                   const float turbulence, string *err) {
    MetaClump cloudClump (1.f);

    // read in meta balls from file, in format:
    // x y z radius
    bool success = false;
    while (geomFile.eof() == false) {
        string tmp;
        string geomtype;
        float x, y, z, r1, r2, r3;

        getline(geomFile, tmp);
        if (istringstream (tmp) >> geomtype)
        {
            if (geomtype == "sphere")
            {
                if (istringstream (tmp) >> geomtype >> x >> y >> z >> r1)
                {
                    cloudClump.addPrim (new Sphere (xyz (x, y, z), r1));
                    cout << "read: " << geomtype << " " << x << " " << y << " " << z << "   " << r1 << endl;
                    success = true;
                }
            }
            else if (geomtype == "ellipse")
            {
                if (istringstream (tmp) >> geomtype >> x >> y >> z >> r1 >> r2 >> r3)
                {
                    cloudClump.addPrim (new Ellipse(xyz (x, y, z), r1, r2, r3));
                    cout << "read: " << geomtype << " " << x << " " << y << " " << z << "   " << r1 << " " << r2 << " " << r3 << endl;
                    success = true;
                }
            }
        }
        /*
        if (istringstream (tmp) >> geomtype >> x >> y >> z >> r) {
            cout << "read: " << geomtype << " " << x << " " << y << " " << z << " " << r << endl;
            if (geomtype == "sphere")
                cloudClump.addPrim (new Sphere (xyz (x, y, z), r));
            else if (geomtype == "ellipse")
                cout << "It's an ellipse\n" << endl;
            success = true;
        }
        */
    }

    if (!success) {
        *err = "No shapes found in input file";
        return false;
    }

    float param1 = densityScale;// 0.3f; // density scaling 
    float param2 = 0.5f; // exponent in pow() function
    float param3 = 0.4f; // blending function value between meta-ball density and product of turbulence density. larger means less turbulent
    float param4 = turbulence; //0.7f; // turbulence in displacing primitive before evaluation of implicit primitives

    cout << "param1:" << param1 << "  param4:" << param4 << endl;

    // find bounds for volume grid
    xyz minPt, maxPt, diff;
    cloudClump.bounds (&minPt, &maxPt, 1.f);

    float offset = param4 * 1.5f; // 1.5? it is mystery
    float offsetHalf = offset * 0.5f;
    minPt.x -= offset;
    minPt.z -= offset;
    maxPt.y += offset;

    diff.x = maxPt.x - minPt.x;
    diff.y = maxPt.y - minPt.y;
    diff.z = maxPt.z - minPt.z;

    cout << "min: " << minPt.x + offset
         << " " << minPt.y - offset
         << " " << minPt.z + offset
         << endl
         << "max: " << maxPt.x + offset
         << " " << maxPt.y - offset
         << " " << maxPt.z + offset
         << endl;

    // fill volume grid

    // calculate number of sample-cells in each direction
    // nudge min/max values so 
    int xCount = (int)ceilf (max(diff.x, 0.f) * samplesPerUnitLength);
    int yCount = (int)ceilf (max(diff.y, 0.f) * samplesPerUnitLength);
    int zCount = (int)ceilf (max(diff.z, 0.f) * samplesPerUnitLength);

    if (xCount <= 0 || yCount <= 0 || zCount <= 0) {
        *err = "metaball dimensionless in some axis";
        return false;
    }

    denseFile << "Volume \"volumegrid\" \"integer nx\" " << xCount
              << " \"integer ny\" " << yCount << " \"integer nz\" " << zCount << endl
              << "\t\"point p0\" [ " << minPt.x + offset
              << " " << minPt.y - offsetHalf
              << " " << minPt.z + offsetHalf
              << " ] \"point p1\" [ " << maxPt.x + offsetHalf
              << " " << maxPt.y - offsetHalf
              << " " << maxPt.z + offsetHalf
              << " ]\n" << "\t\"float density\" [\n";


    for (int iZ = 0; iZ < zCount; ++iZ) {
        float zPcnt = (float)iZ / (zCount - 1);
        float zPos = zPcnt * diff.z + minPt.z;

        for (int iY = 0; iY < yCount; ++iY) {
           float yPcnt = (float)iY / (yCount - 1);
            float yPos = yPcnt * diff.y + minPt.y;

            for (int iX = 0; iX < xCount; ++iX) {
                float xPcnt = (float)iX / (xCount - 1);
                float xPos = xPcnt * diff.x + minPt.x;
                float val = 0;
                /*
                for (SphereVec::iterator ballIt = cloudBalls.begin();
                                         ballIt != cloudBalls.end ();
                                         ++ballIt) {
                    if ((gridPt - ballIt->pos).length2() <= ballIt->r2)
                        val = 0.05f;
                    //cout << (gridPt - ballIt->pos).x << "  " << (gridPt - ballIt->pos).y << "  " << (gridPt - ballIt->pos).z << endl;
                }
                */
                if (iZ == 36 && iY == 36 && iX == 68)
                    int a = 0;

                //val = cloudClump.evaluate (xyz (xPos, yPos, zPos));
                val = cumulus_density (
                            xyz (xPcnt, yPcnt, zPcnt) * RAND_GRID,      // location of point in cloud space 
                            xyz (xPos, yPos, zPos),    // location of point in world space 
                            cloudClump,// volume
                            param1,  // density scaling 
                            param2,  // exponent in pow() function
                            param3,  // blending function value between meta-ball density and product of turbulence density. larger means less turbulent
                            param4); // turbulence in displacing primitive before evaluation of implicit primitives

                denseFile << val << " ";
            } // end x
            denseFile << endl;
        } // end y
        denseFile << endl;
        cout << (int)((((float)iZ + 1) / zCount) * 100) << "% ";
    } // end z 
    denseFile << "]\n";
    cout << endl;

    return true;
}


void main (int argc, char **args) {
    if (argc != 6) {
        cout << "proper usage: CloudGen metaballsIN.txt densityOUT.pbrt samplesPerUnitLength densityScale turbulence\n";
        exit(0);
    }

    srand (0);

    // open metaball description file for input
    ifstream geomFile;
    string geomFilename(args[1]);
    geomFile.open (geomFilename.c_str());
    if (geomFile.is_open() == false) {
        cerr << "ERROR: cannot open " << geomFilename << endl;
        exit (1);
    }

    // open density file for output
    ofstream denseFile;
    string denseFilename(args[2]);
    denseFile.open (denseFilename.c_str());
    if (denseFile.is_open() == false) {
        geomFile.close();
        cerr << "ERROR: cannot open " << denseFilename << endl;
        exit (1);
    }

    // read other values
    float samplesPerUnitLength = (float)atof (args[3]);
    float densityScale = (float)atof (args[4]);
    float turbulence = (float)atof (args[5]);

    denseFile << "# Created by CloudGen, based upon " << geomFilename 
              << " with samplesPerUnitLength:" << samplesPerUnitLength
              << " densityScale:" << densityScale
              << " turbulence:" << turbulence << endl;

    cout << "samplesPerUnitLength:" << samplesPerUnitLength << "  densityScale:" << densityScale << "  turbulence:" << turbulence << endl;

    // generate noise for clouds
    generate_noise();

    // process files
    string error;
    if (processFiles (geomFile, denseFile, samplesPerUnitLength, densityScale, turbulence, &error) == false)
        cerr << "ERROR: " << error << endl;

    geomFile.close();
    denseFile.close();
}